"use strict";

/**
 * Rax Document Plugin
 * Generate HTML by compiling document/index.js and App Shell
 *
 */
var fs = require('fs');

var minify = require('html-minifier').minify;

var _require = require('../env'),
    isProductionMode = _require.isProductionMode;

var mkTempDir = require('../mkTempDir');

var getSPAPagesConfig = require('../getSPAPagesConfig');

var AppShellHandler = require('../AppShellHandler');

var DocumentHandler = require('../DocumentHandler');

var PLUGIN_NAME = 'RaxDocumentPlugin';

var RaxDocumentPlugin =
/*#__PURE__*/
function () {
  function RaxDocumentPlugin(options) {
    this.options = options;
  }

  var _proto = RaxDocumentPlugin.prototype;

  _proto.apply = function apply(compiler) {
    var _this$options = this.options,
        appConfig = _this$options.appConfig,
        pathConfig = _this$options.pathConfig; // Mark optimization function points on PWA

    var withSkeleton = false;
    var withSPA = !!appConfig.spa;
    var withSSR = !!appConfig.ssr;
    var withAppShell = fs.existsSync(pathConfig.appShell);
    var withDocumentJs = fs.existsSync(pathConfig.appDocument) || !fs.existsSync(pathConfig.appHtml); // String template for injecting HTML

    var appShellTemplate = '';
    var skeletonTemplate = ''; // Mark the current environment

    var isProductionLikeMode = isProductionMode(compiler);
    var appShellHandler = new AppShellHandler({
      appDirectory: pathConfig.appDirectory,
      appShell: pathConfig.appShell
    });
    var documentHandler = new DocumentHandler({
      appDocument: pathConfig.appDocument,
      appHtml: pathConfig.appHtml,
      appDirectory: pathConfig.appDirectory
    }); // Make temp directory

    mkTempDir(pathConfig.appDirectory);

    if (withSPA) {
      var pagesConfig = getSPAPagesConfig(appConfig, pathConfig); // Prepare the skeleton diagram code, match the routing information when the page is initialized,
      // and insert the skeleton diagram of the corresponding page into the blank area of the page

      Object.keys(pagesConfig).forEach(function (pageName) {
        if (pagesConfig[pageName].skeleton) {
          withSkeleton = true;

          if (pageName === 'index') {
            skeletonTemplate += "\n              if (isPathMatched(" + pagesConfig[pageName]._regexp + ", \"" + (withSSR ? 'history' : 'hash') + "\") || (" + withSSR + " && pathname === '/') || (" + !withSSR + " && hash === '/')) {\n                document.getElementById(\"" + (withAppShell ? 'root-page' : 'root') + "\").innerHTML = '<img src=\"" + pagesConfig[pageName].skeleton + "\"/>';\n              }\n            ";
          } else {
            skeletonTemplate += "\n              if (isPathMatched(" + pagesConfig[pageName]._regexp + ", \"" + (withSSR ? 'history' : 'hash') + "\")) {\n                document.getElementById(\"" + (withAppShell ? 'root-page' : 'root') + "\").innerHTML = '<img src=\"" + pagesConfig[pageName].skeleton + "\"/>';\n              }\n            ";
          }
        }
      });

      if (withSkeleton) {
        skeletonTemplate = "\n          var pathname = window.location.pathname + window.location.search;\n          var hash = window.location.hash.replace(\"#\", \"\");\n          var isPathMatched = function (regexp, type) {\n            return \"hash\" === type ? regexp.test(hash) : \"history\" === type && regexp.test(pathname);\n          };\n        " + skeletonTemplate;
      }
    }
    /**
     * Replace code before Build
     * 1. Custom document/index.js compilation.
     * 2. Compile the App Shell file. The string node after render string is inserted into HTML.
     */


    compiler.hooks.beforeCompile.tapAsync(PLUGIN_NAME, function (compilationParams, callback) {
      // callback must run only once
      var callbackCalled = false;

      var runCallback = function runCallback() {
        if (callbackCalled) return;
        callbackCalled = true;
        callback();
      };

      if (!withSSR && (withAppShell || withDocumentJs)) {
        if (withAppShell && withDocumentJs) {
          var next = function next(type) {
            if (type === 'appShell') {
              appShellReady = true;
            }

            if (type === 'document') {
              documentReady = true;
            }

            if (appShellReady && documentReady) {
              runCallback();
            }
          };

          var appShellReady = false;
          var documentReady = false;
          withAppShell && appShellHandler.build(next);
          withDocumentJs && documentHandler.build(next);
        } else {
          withAppShell && appShellHandler.build(runCallback);
          withDocumentJs && documentHandler.build(runCallback);
        }
      } else {
        runCallback();
      }
    });
    /**
     * Generate code that really works
     * 1. Insert the App Shell file into HTML
     * 2. Custom document/index.js handles HTML files into containers
     */

    compiler.hooks.emit.tapAsync(PLUGIN_NAME, function (compilation, callback) {
      if (!withSSR) {
        // Process app shell
        if (withAppShell) {
          appShellTemplate = appShellHandler.getContent();
        } // Process html


        var htmlCode;
        var document = documentHandler.getDocument(compilation, appShellTemplate, skeletonTemplate, appConfig.title);

        if (isProductionLikeMode) {
          // https://github.com/kangax/html-minifier#options-quick-reference
          htmlCode = minify(document.html, {
            minifyJS: true,
            collapseWhitespace: true,
            removeComments: true,
            removeRedundantAttributes: true,
            removeScriptTypeAttributes: true,
            removeStyleLinkTypeAttributes: true,
            useShortDoctype: true
          });
        } else {
          htmlCode = document.html;
        } // add index.html


        compilation.fileDependencies.add(document.path);
        compilation.assets['index.html'] = {
          source: function source() {
            return htmlCode;
          },
          size: function size() {
            return htmlCode.length;
          }
        };
      }

      callback();
    });
  };

  return RaxDocumentPlugin;
}();

module.exports = RaxDocumentPlugin;